#ifndef __XMLIO_GROUP_TEMPLATE__
#define __XMLIO_GROUP_TEMPLATE__

namespace XMLIOSERVER
{
   template <class T, class U>
      class GroupTemplate : public ObjectTemplate<GroupTemplate<T, U> >, public U
   {
      public:

         GroupTemplate(void) : ObjectTemplate<GroupTemplate<T, U> >(),  U(), childList(), groupList()
         {/* Ne rien faire de plus */}
         GroupTemplate(const string& _id) : ObjectTemplate<GroupTemplate<T, U> >(_id), U(), childList(), groupList()
         {/* Ne rien faire de plus */}

         /// Pour les groupes d'objets enfants ///

         static string GetName(void) {return (T::GetName().append("_group")); }
         static string GetDefName(void) {return (T::GetName().append("_definition")); }

         virtual bool hasChild(void) const { return ((getNbGroup() + getNbChild())>0); }
         virtual void printChild(ostream& out) const
         {
            // Ecriture des sous-groupes.
            for(unsigned int i = 0; i < groupList.getVector().size() ; i++)
               out << *(groupList.getVector()[i])  << std::endl;
            // Ecriture des enfants.
            for(unsigned int i = 0; i < childList.getVector().size() ; i++)
               out << *(childList.getVector()[i]) << std::endl;
         }

         virtual void resolveDescInheritance(const AttributRegistrar* _parent = 0)
         {
            const vector<T*>&  childvect = childList.getVector();
            const vector<GroupTemplate<T, U>*>& groupvect = groupList.getVector();

            // On complète les propres attributs du groupe.
            if (_parent!= NULL) AttributRegistrar::addAttributes(*_parent);

            for(unsigned int i = 0; i < childvect.size() ; i++)
            // on complète les attributs des enfants.
               childvect[i] -> resolveDescInheritance(this);
            for(unsigned int i = 0; i < groupvect.size() ; i++)
            // on complète les attributs des groupes enfants.
               groupvect[i] -> resolveDescInheritance(this);
         }

         GroupTemplate<T, U>& createGroup(const string _id) throw (XMLIOUndefinedValueException)
         {
            GroupTemplate<T, U> &obj = GroupTemplate<T, U>::CreateObject(_id);
            groupList.addObject(&obj);

            return (GroupTemplate<T, U>::GetObject(_id));
         }

         GroupTemplate<T, U>& createGroup(void)
         {
            GroupTemplate<T, U>& obj = GroupTemplate<T, U>::CreateObject();
            groupList.addObject(&obj);

            return (obj);
         }

         GroupTemplate<T, U>& getGroup(const string _id) throw (XMLIOUndefinedValueException) { return (*groupList[_id]); }
         bool hasGroup(const string _id) { return (groupList.hasMappedValue(_id)); }

         const StrHashMap<GroupTemplate<T, U>* >& getGroupList(void) { return (groupList); }

         size_t getNbGroup() const {return (groupList.getVectorSize()); }

         /// Pour les objets enfants ///

         T& createChild(const string _id) throw (XMLIOUndefinedValueException)
         {
            T& obj = ObjectTemplate<T>::CreateObject(_id);
            childList.addObject(&obj);
            return (obj);
         }

         T& createChild(void)
         {
            T& obj = ObjectTemplate<T>::CreateObject();
            childList.addObject(&obj);
            return (obj);
         }

         T& getChild(const string _id) throw (XMLIOUndefinedValueException) { return (*childList[_id]); }
         bool hasChild(const string _id) { return (childList.hasMappedValue(_id)); }

         const StrHashMap<T*>& getCurrentListChild(void) { return (childList); }

         size_t getNbChild() const {return (childList.getVectorSize()); }

         virtual ~GroupTemplate()
         {
            for (unsigned int i = 0; i < childList.getVector().size(); i++)
               delete childList.getVector()[i];
            for (unsigned int i = 0; i < groupList.getVector().size(); i++)
               delete groupList.getVector()[i];
         }

      protected:

         template <class V> V* createGroupAndParse(THashAttributes& attributes, XMLNode& _node)
         {
            V* instance_ptr = NULL;
            if (attributes.end() != attributes.find("id"))
            {// Si l'identifiant est défini.
               if (V::HasObject(attributes["id"]))
                  WARNING("Dans le context actuel, une instance de type "+V::GetName()+" nommée "+attributes["id"]+" existe déjà, le second fera référence au premier par défaut !"); // TODO TODO
               instance_ptr = (V*)(&createGroup(attributes["id"]));
               instance_ptr->parse(_node);
            }
            else
            {// Si l'identifiant n'est pas défini.
               instance_ptr = (V*)(&createGroup());
               instance_ptr->parse(_node);
            }
            return (instance_ptr);
         }

         template <class V> V* createChildAndParse(THashAttributes& attributes, XMLNode& _node)
         {
            V* instance_ptr = NULL;
            if (attributes.end() != attributes.find("id"))
            {// Si l'identifiant est défini.
               if (V::HasObject(attributes["id"]))
                  WARNING("Dans le context actuel, une instance de type "+V::GetName()+" nommée "+attributes["id"]+" existe déjà, le second fera référence au premier par défaut !"); // TODO TODO
               instance_ptr = (V*)(&createChild(attributes["id"]));
               instance_ptr->parse(_node);
            }
            else
            {// Si l'identifiant n'est pas défini.
               instance_ptr = (V*)(&createChild());
               instance_ptr->parse(_node);
            }
            return (instance_ptr);
         }

         StrHashMap<T> childList;
         StrHashMap<GroupTemplate<T, U> > groupList;

   }; // class GroupTemplate



}// namespace XMLIOSERVER


#endif // __XMLIO_GROUP_TEMPLATE__
